@@ -49,6 +49,11 @@ support for the British -ise suffix.
 
  2000.08.25 - 2.11 Added stemming cache
 
+ 2000.09.14 - 2.12 Fixed *major* :( implementation error of Porter's algorithm
+              Error was entirely my fault - I completely forgot to include
+              rule sets 2,3, and 4 starting with Lingua::Stem 0.30. 
+              -- Benjamin Franz
+
 =cut
 
 #######################################################################
@@ -65,7 +70,7 @@ BEGIN {
     @EXPORT_OK   = qw (stem clear_stem_cache stem_caching);
     %EXPORT_TAGS = ();
 }
-$VERSION = "2.11";
+$VERSION = "2.12";
 
 my $Stem_Caching  = 0;
 my $Stem_Cache    = {};
@@ -230,19 +235,25 @@ sub stem {
         #  (where the eval ignores undefined subroutines) by the much faster
         #    eval { &{ 'S2' . substr( $_, 0, 3 ) } };
         #  But the following is slightly faster still:
+
+        { 
+            no strict 'refs';
+            
+            my $sub;
     
-        my $sub;
+            #  Step 3: double and triple suffices, etc (part 2)
+
+            &$sub if defined &{ $sub = 'S2' . substr( $_, 0, 3 ) };
     
-        &$sub if defined &{ $sub = 'S2' . substr( $_, 0, 3 ) };
+            #  Step 3: double and triple suffices, etc (part 2)
     
-        #  Step 3: double and triple suffices, etc (part 2)
+            &$sub if defined &{ $sub = 'S3' . substr( $_, 0, 3 ) };
     
-        &$sub if defined &{ $sub = 'S3' . substr( $_, 0, 3 ) };
-    
-        #  Step 4: single suffices on polysyllables
-    
-        &$sub if defined &{ $sub = 'S4' . substr( $_, 0, 2 ) };
+            #  Step 4: single suffices on polysyllables
     
+            &$sub if defined &{ $sub = 'S4' . substr( $_, 0, 2 ) };
+   
+        }
         #  Step 5a: tidy up final e -- probate->probat, rate->rate; cease->ceas
     
         m!^e! && ( s!^e($syl$syl)!$1!o ||
@@ -269,6 +280,200 @@ sub stem {
     return $words;
 }
 
+##############################################################
+# Rule set 2
+
+sub S2lan {
+    #  SYLational -> SYLate,	SYLtional -> SYLtion
+    s!^lanoita($syl)!eta$1!o || s!^lanoit($syl)!noit$1!o;
+}
+
+sub S2icn {
+    #  SYLanci -> SYLance, SYLency ->SYLence
+    s!^icn([ae]$syl)!ecn$1!o;
+}
+
+sub S2res {
+    #  SYLiser -> SYLise **
+    &S2rez;
+}
+
+sub S2rez {
+    #  SYLizer -> SYLize
+    s!^re(.)i($syl)!e$1i$2!o;
+}
+
+sub S2ilb {
+    #  SYLabli -> SYLable, SYLibli -> SYLible ** (e.g. incredibli)
+    s!^ilb([ai]$syl)!elb$1!o;
+}
+
+sub S2ill {
+    #  SYLalli -> SYLal
+    s!^illa($syl)!la$1!o;
+}
+
+sub S2ilt {
+    #  SYLentli -> SYLent
+    s!^iltne($syl)!tne$1!o
+}
+
+sub S2ile {
+    #  SYLeli -> SYLe
+    s!^ile($syl)!e$1!o;
+}
+
+sub S2ils {
+    #  SYLousli -> SYLous
+    s!^ilsuo($syl)!suo$1!o;
+}
+
+sub S2noi {
+    #  SYLization -> SYLize, SYLisation -> SYLise**, SYLation -> SYLate
+    s!^noita([sz])i($syl)!e$1i$2!o || s!^noita($syl)!eta$1!o;
+}
+
+sub S2rot {
+    #  SYLator -> SYLate
+    s!^rota($syl)!eta$1!o;
+}
+
+sub S2msi {
+    #  SYLalism -> SYLal
+    s!^msila($syl)!la$1!o;
+}
+
+sub S2sse {
+    #  SYLiveness  -> SYLive, SYLfulness -> SYLful, SYLousness -> SYLous
+    s!^ssen(evi|luf|suo)($syl)!$1$2!o;
+}
+
+sub S2iti {
+    #  SYLaliti -> SYLal, SYLiviti -> SYLive, SYLbiliti ->SYLble
+    s!^iti(la|lib|vi)($syl)! ( $1 eq 'la' ? 'la' : $1 eq 'lib' ? 'elb' : 'evi' )
+	. $2 !eo;
+}
+
+##############################################################
+# Rule set 3
+
+sub S3eta {
+    #  SYLicate -> SYLic
+    s!^etaci($syl)!ci$1!o;
+}
+
+sub S3evi {
+    #  SYLative -> SYL
+    s!^evita($syl)!$1!o;
+}
+
+sub S3ezi
+{
+    #  SYLalize -> SYLal
+    s!^ezila($syl)!la$1!o;
+}
+
+sub S3esi {
+    #  SYLalise -> SYLal **
+    s!^esila($syl)!la$1!o;
+}
+
+sub S3iti {
+    #  SYLiciti -> SYLic
+    s!^itici($syl)!ci$1!o;
+}
+
+sub S3lac {
+    #  SYLical -> SYLic
+    s!^laci($syl)!ci$1!o;
+}
+sub S3luf {
+    #  SYLful -> SYL
+    s!^luf($syl)!$1!o;
+}
+
+sub S3sse {
+    #  SYLness -> SYL
+    s!^ssen($syl)!$1!o;
+}
+
+##############################################################
+# Rule set 4
+
+sub S4la {
+    #  SYLSYLal -> SYLSYL
+    s!^la($syl$syl)!$1!o;
+}
+
+sub S4ec {
+    #  SYLSYL[ae]nce -> SYLSYL
+    s!^ecn[ae]($syl$syl)!$1!o;
+}
+
+sub S4re {
+    #  SYLSYLer -> SYLSYL
+    s!^re($syl$syl)!$1!o;
+}
+
+sub S4ci {
+    #  SYLSYLic -> SYLSYL
+    s!^ci($syl$syl)!$1!o;
+}
+
+sub S4el {
+    #  SYLSYL[ai]ble -> SYLSYL
+    s!^elb[ai]($syl$syl)!$1!o;
+}
+
+sub S4tn {
+    #  SYLSYLant -> SYLSYL, SYLSYLe?ment -> SYLSYL, SYLSYLent -> SYLSYL
+    s!^tn(a|e(me?)?)($syl$syl)!$3!o;
+}
+sub S4no {
+    #  SYLSYL[st]ion -> SYLSYL[st]
+    s!^noi([st]$syl$syl)!$1!o;
+}
+
+sub S4uo {
+    #  SYLSYLou -> SYLSYL e.g. homologou -> homolog
+    s!^uo($syl$syl)!$1!o;
+}
+
+sub S4ms {
+    #  SYLSYLism -> SYLSYL
+    s!^msi($syl$syl)!$1!o;
+}
+
+sub S4et {
+    #  SYLSYLate -> SYLSYL
+    s!^eta($syl$syl)!$1!o;
+}
+
+sub S4it {
+    #  SYLSYLiti -> SYLSYL
+    s!^iti($syl$syl)!$1!o;
+}
+
+sub S4su {
+    #  SYLSYLous -> SYLSYL
+    s!^suo($syl$syl)!$1!o;
+}
+
+sub S4ev { 
+    #  SYLSYLive -> SYLSYL
+    s!^evi($syl$syl)!$1!o;
+}
+
+sub S4ez {
+    #  SYLSYLize -> SYLSYL
+    s!^ezi($syl$syl)!$1!o;
+}
+
+sub S4es {
+    #  SYLSYLise -> SYLSYL **
+    s!^esi($syl$syl)!$1!o;
+}
+
 ##############################################################
 
 =over 4
